<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
      xmlns:xs="http://www.w3.org/2001/XMLSchema"
      exclude-result-prefixes="xs"
      version="2.0">
    
   <xsl:param name="domainPrefix">lexs</xsl:param>
    <xsl:param name="runMode">ignore</xsl:param><!-- all, default, ignore, UID, duplicates -->

    <xsl:output name="checkDups" method="text" indent="yes" omit-xml-declaration="yes"/>
    <xsl:output name="XMLout" indent="yes" method="xml"/>
    
    <xsl:variable name="version">1.01</xsl:variable>
    
    <xsl:variable name="everything" select="/"></xsl:variable>
    
    <xsl:variable name="duplicates">
        <xsl:apply-templates select="ITEMSlookup" mode="duplicates"/>
    </xsl:variable>
    
      <xsl:template match="/">
         
          <xsl:element name="ITEMSlookup">
            <xsl:comment> Dictionary de-duplicate tool, version <xsl:value-of select="$version"/>.</xsl:comment>
            <xsl:apply-templates select="ITEMSlookup"/>
          </xsl:element>
          
          <xsl:if test="$runMode='duplicates' or($runMode='all') or($runMode='UID')">
              <xsl:result-document format="checkDups" indent="yes"
                  href="testDups.xml">
                      <xsl:apply-templates select="ITEMSlookup" mode="DupChk"/>
              </xsl:result-document>
          </xsl:if>
          
      </xsl:template>
    
    <xsl:template match="Property">
        <xsl:variable name="thisContext" select="@context"/>
        <xsl:choose>   <!-- duplicate check -->
                <xsl:when test="not($runMode='ignore') and count($duplicates/duplicate[@context=$thisContext]) &gt; 0 and(position() &gt; $duplicates/duplicate[@context=$thisContext][1]/@position)">
                        <xsl:comment>duplicate item skipped: <xsl:value-of select="concat(position(),' - ',@CCTS,' - ',$thisContext)"/></xsl:comment>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:element name="Property">
                        <xsl:apply-templates select="attribute::*">
                            <xsl:with-param name="UIDvalue" select="generate-id()"/>
                            <xsl:with-param name="itemName" select="@item"/>
                        </xsl:apply-templates>
                    </xsl:element>
                </xsl:otherwise>
            </xsl:choose>
    </xsl:template>
    
    <xsl:template match="Property" mode="DupChk">
        <xsl:variable name="thisUID" select="@UID"/>
        <xsl:variable name="thisContext" select="@context"/>
        <xsl:variable name="depthChk" select="tokenize($thisContext,'/')"/>
        <xsl:variable name="depthCount"><xsl:for-each select="$depthChk"><counter/></xsl:for-each></xsl:variable>
        <xsl:if test="$runMode='UID' or($runMode='all')">
            <xsl:if test="count($everything/ITEMSlookup/Property[@UID=$thisUID]) &gt; 1"><xsl:value-of select="@UID"/>,<xsl:value-of select="@context"/>,<xsl:value-of select="@item"/>,UID</xsl:if>
        </xsl:if>
        <xsl:if test="$runMode='duplicates' or($runMode='all')">
            <xsl:choose>
                <xsl:when test="count($everything/ITEMSlookup/Property[@context=$thisContext]) &gt; 1"><xsl:value-of select="count($depthCount/counter)"/> | <xsl:value-of select="@context"/>,<xsl:value-of select="@UID"/>,<xsl:value-of select="@item"/>, context</xsl:when>
                <xsl:otherwise/>
            </xsl:choose>
        </xsl:if>
        <xsl:apply-templates select="Property/."/>
    </xsl:template>
    
    <xsl:template match="Property" mode="duplicates">
        <xsl:variable name="thisContext" select="@context"/>
            <xsl:choose>
                <xsl:when test="count($everything/ITEMSlookup/Property[@context=$thisContext]) &gt; 1">
                    <duplicate position="{position()}" context="{@context}" item="{@item}"/>
                </xsl:when>
                <xsl:otherwise/>
            </xsl:choose>
        <xsl:apply-templates select="Property/."/>
    </xsl:template>

    <xsl:template match="comment()"><xsl:copy/></xsl:template>
    
    <xsl:template match="@*">
        <xsl:param name="UIDvalue"/>
        <xsl:param name="itemName"/>
        <xsl:variable name="attrName" select="name()"/>
        <xsl:choose>
            <xsl:when test="$attrName='item'  and(not(contains(.,':')))"><!-- fix if default namespace -->
                <xsl:attribute name="item" select="concat($domainPrefix,':',$itemName)"/>
            </xsl:when>
            <xsl:when test="$attrName='UID'">
                <xsl:attribute name="UID" select="if(contains($itemName,':')) then concat(substring-before($itemName,':'),'-',$UIDvalue) else concat($domainPrefix,'-',$UIDvalue)"/>
            </xsl:when>
            <xsl:when test="$attrName='context'"><!-- fix if default namespace -->
                <xsl:variable name="newContext"><xsl:call-template name="fix-context"><xsl:with-param name="context" select="."/></xsl:call-template></xsl:variable>
                <xsl:attribute name="context" select="$newContext"/>
            </xsl:when>
            <xsl:otherwise><xsl:copy/></xsl:otherwise>
        </xsl:choose>
    </xsl:template>

    <xsl:template name="fix-context">
        <xsl:param name="context"/>
        <xsl:for-each select="tokenize($context,'/')">
            <xsl:choose>
                <xsl:when test="contains(.,':')"><xsl:value-of select="concat('/',.)"/></xsl:when>
                <xsl:when test="string-length(.) &lt; 1"/>
                <xsl:otherwise><xsl:value-of select="concat('/',$domainPrefix,':',.)"/></xsl:otherwise>
            </xsl:choose>
        </xsl:for-each>
    </xsl:template>

</xsl:stylesheet>
