<?xml version="1.0" encoding="UTF-8"?>
<!-- Add group attribute to Properties element : 6th August, 2009 -->
<!-- Update attribute processing so list of attributes is dynamic from input : 2nd June, 2010 -->
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
      xmlns:xs="http://www.w3.org/2001/XMLSchema"
      exclude-result-prefixes="xs"
      version="2.0">
    
    <xsl:param name="lookup">EML-v5-dictionary-stuff.xml</xsl:param>
    
    <xsl:variable name="eml_lookup" select="document($lookup)"/>
    <xsl:output method="xml" indent="yes"></xsl:output>
    
<xsl:template match="/">
    <xsl:message>Lookup items:[<xsl:value-of select="count($eml_lookup/EML-doc/Entry)"/>]</xsl:message>
    <ITEMSlookup>
        <xsl:comment><xsl:value-of select="descendant::comment()"/></xsl:comment>
    <xsl:apply-templates select="//Property"/>
    </ITEMSlookup>
</xsl:template>
    
    <xsl:template match="Property">
        <xsl:variable name="this" select="@item"/>
        <xsl:variable name="defn" select="@definition"/>
        <xsl:variable name="definition">
            <xsl:choose>
                <xsl:when test="string-length(@definition) &gt; 0">
                    <xsl:value-of select="@definition"/>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:variable name="DEFN" select="$eml_lookup/EML-doc/Entry[@Name=$this]"/>
                    <xsl:value-of select="$DEFN[1]/@Definition"/>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:variable>
        <xsl:variable name="definitionAction">
        <xsl:choose>
            <xsl:when test="contains(@item,':')"/>
            <xsl:when test="string-length(@definition) &gt; 0">
                
                <xsl:variable name="check">
                    <xsl:choose>
                        <xsl:when test="count($eml_lookup/EML-doc/Entry[@Name=$this]) = 0">OK</xsl:when>    
                        <xsl:when test="$eml_lookup/EML-doc/Entry[@Name=$this and(@Definition=$defn)]">duplicate</xsl:when>
                        <xsl:otherwise>DIFFERENT</xsl:otherwise>
                   </xsl:choose>
                </xsl:variable>
                <xsl:value-of select="$check"/>
                <xsl:message>Verify - <xsl:value-of select="concat(@item,' ',$check)"/></xsl:message>
            </xsl:when>
            <xsl:otherwise>
               <xsl:variable name="check">
                   <xsl:choose>
                   <xsl:when test="$eml_lookup/EML-doc/Entry[@Name=$this and (string-length(@Definition) &gt; 0)]">found</xsl:when>
                    <xsl:when test="$eml_lookup/EML-doc/Entry[@Name=$this and (string-length(@Definition) &lt; 1)]">EMPTY</xsl:when>
                    <xsl:when test="count($eml_lookup/EML-doc/Entry[@Name=$this]) = 0">MISSING</xsl:when>
                    <xsl:otherwise/>
                   </xsl:choose>
               </xsl:variable>
                <xsl:value-of select="$check"/>
                <xsl:message>Find - <xsl:value-of select="concat(@item,' ',$check)"/></xsl:message>
            </xsl:otherwise>
        </xsl:choose>
        </xsl:variable>
                
        <Property>
            <xsl:for-each select="@*">
                <xsl:choose>
                    <xsl:when test="name()='definition'">
                        <xsl:choose>
                            <xsl:when test="$definitionAction='found'"><xsl:attribute name="{name()}" select="$definition"/></xsl:when>
                            <xsl:when test="$definitionAction='MISSING'"><xsl:attribute name="{name()}" select="."/></xsl:when>
                            <xsl:when test="$definitionAction='DIFFERENT'"><xsl:attribute name="{name()}" select="."/></xsl:when>
                            <xsl:otherwise><xsl:attribute name="{name()}" select="."/></xsl:otherwise>
                        </xsl:choose>
                    </xsl:when>
                    <xsl:otherwise><xsl:attribute name="{name()}" select="."/></xsl:otherwise>
                </xsl:choose>  
            </xsl:for-each>
        </Property> 
        <xsl:if test="descendant::comment()"><xsl:comment><xsl:value-of select="descendant::comment()"/></xsl:comment></xsl:if>
        <xsl:if test="comment()"><xsl:comment><xsl:value-of select="comment()"/></xsl:comment></xsl:if>
    </xsl:template>
    
</xsl:stylesheet>
